#!/bin/sh
# Copyright(c) 2007 Red Hat Middleware, LLC,
# and individual contributors as indicated by the @authors tag.
# See the copyright.txt in the distribution for a
# full listing of individual contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library in the file COPYING.LIB;
# if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
#
# @author Jean-Frederic Clere
#

# Download the sources. It guess how to do it.
# $1: URL.
# $1: Back URL. (archive for apache)
downloadtaredsrc()
{
  URL=$1
  URLBACK=$2
  if [ -z $URLBACK ]; then
    URLBACK=$URL
  fi
  wget --tries=3 --retry-connrefused $URL
  if [ $? -ne 0 ]; then
    echo "downloadtaredsrc Retrying on $URLBACK"
    wget --tries=3 --retry-connrefused $URLBACK
    if [ $? -ne 0 ]; then
      echo "downloadtaredsrc $URLBACK failed"
      return 1
    fi
    URL=$URLBACK
  fi
  fname=`basename $URL`
  case ${fname} in
    *.tar.gz)
      gunzip -c $fname | tar -xf -
      ;;
    *.zip)
      unzip -q -o $fname
      ;;
  esac
  rm -f $fname
  return 0
}

# Check and download
# $1 directory of the checkout directory
# $2 url for the download
# $3 backup url for the download
ckeckdownload()
{
  src_dir=$1
  src_url=$2
  src_url_back=$3
  src_patching=$4
  cd $build_cache_dir
  if [ ! -d ${src_dir} ]; then
    downloadtaredsrc $src_url $src_url_back
    if [ $? -ne 0 ]; then
      return 1
    fi
    if $src_patching; then
      $build_top/util/applypatch.sh $build_top $src_dir false
    fi
  fi
  echo "Copying ${src_dir} to ${package_src_dir}/srclib"
  cp -rp ${src_dir} ${package_src_dir}/srclib
  cd $build_top
}

# ckeckdownload sub shell version.
# Check and download
# $1 cache directory.
# $2 directory where the sources are located.
# $3 build_top. Where we start the build.sh
# $4 directory of the checkout directory
# $5 url for the download
# $6 backup url for the download
build_cache_dir=$1
package_src_dir=$2
build_top=$3
src_dir=$4
url=$5
urlback=$6
patching=$7

echo ""
echo "Running `basename $0` $LastChangedDate: 2014-01-27 09:32:17 -0500 (Mon, 27 Jan 2014) $"
echo ""
echo "Started : `date`"
echo "Params  : $@"
echo ""

echo "url: $url urlback: $urlback"
if [ "x$patching" = "x" ]; then
  patching=false
fi
echo "patching: $patching"

# Call the internal functions.
ckeckdownload $src_dir $url $urlback $patching
if [ $? -ne 0 ]; then
  echo "ckeckdownload src_dir: $src_dir url: $url urlback: $urlback FAILED!"
  exit 1
fi
exit 0
